#! /usr/bin/perl
#* ============================================================
# * File        : retrovision/catlist.pl
# * Author      : Eric
# * Date        : 2007-12-07
# * Description : parses mainpage categories
# * ============================================================ */

use English;
use XML::DOM;
use HTML::Entities;

#------------------------------------------------------------------------------
# Init
#------------------------------------------------------------------------------

&read_parse();    # get commandline parameters into @in
$source = $in[0]; # source filename from command line

my $doc = XML::DOM::Document->new;
my $head = $doc->createXMLDecl ('1.0');
my $root = $doc->createElement('items');

sub newNode
{
  local $name  = shift;
  local $value = shift;
  local $node = $doc->createElement($name);
  local $text = $doc->createTextNode($value);
  $node->appendChild($text);
  
  return $node;
}

sub checkNode
{
   local $node = shift;   
   return ( $node && !($node =~ /^HASH/) )  # deref empty hash element
}

sub addMetaNode
{
  local $node  = shift;
  local $name  = shift;
  local $type  = shift;
  
  if ( checkNode($node) )
  {
   if ($type eq '')
   {
     $type = "inline";
     if ($node =~ /\n/)           { $type = "text" }
     if ($node =~ /<.+>.+<\/.+>/) { $type = "html" }
   }
   
   $meta = $doc->createElement('meta');
   $meta->appendChild( newNode('name'   , $name) );
   $meta->appendChild( newNode('content', $node) );
   $meta->appendChild( newNode('viewer' , $type) );
   $item->appendChild( $meta );
  }
}

#------------------------------------------------------------------------------
# read file into $data
#------------------------------------------------------------------------------

$datafile = $source;
open( INFO, "<$datafile" );      # Open file for reading
undef $/;
$data = <INFO>;                 # Read all
close(INFO);
#------------------------------------------------------------------------------
# Parse playlist
#------------------------------------------------------------------------------



# in http://freedocumentaries.org/index.php?ct=26 (selected category on freedocumentaries.org)

#<span class="style20"><a href=film.php?id=114 class="lefttit">Invisible Children</a></span><br>
#                    <img src="images/transparente.gif" width="10" height="10"><br>
#                    Watch one of the biggest stories that you have never heard of. Discover a war which few have heard of; a war in which the rebels are ruthless murderers of civilians yet are hard to hate.  This is because they are only children.
#                    <br>
#
#                  <img src="images/transparente.gif" width="10" height="10">                  </span></td>

# substitute [number] with 114:
# http://freedocumentaries.org/film.php?id=[number]
# parse resulting page with film.pl

# in http://freedocumentaries.org/film.php?id=171 find line:
# <a href="javascript: void launchFilm('1033');void window.open('pop.php', '', 'width=350, height=180');" class="right"><span class="style5-watch">Watch Film Now!</span></a>

# substitue [number] with 1033 from previous page results in url:
# http://freedocumentaries.org/theatre.php?id=[number]
# that page should be handed to vid.pl

sub dump_lines
{
  # get rid of newlines
  $data =~ s/\n/ /g;
  # multiple spaces
  $data =~ s/\s\s+/ /g;
  # format station links
  $data = decode_entities($data);

  while ($data =~ m/<a href=film.php\?id=(\d+)\sclass="lefttit">([^<]+)<\/a><\/span><br>.*?<br>([^<]*)</gi)
  {
     $id = $1;
     $name = $2;
     $descr = $3;
     
     $url = "http://freedocumentaries.org/film.php?id=$id";
     #$name =~ s/\%([A-Fa-f0-9]{2})/pack('C', hex($1))/seg;

     $item = $doc->createElement('item');
     $root->appendChild($item);
     $item->appendChild( newNode('name', "$name") );
     $item->appendChild( newNode('url', $url) );
     $item->appendChild( newNode('descr', $descr) );
     $item->appendChild( newNode('handler', "freedocumentaries/film") );

     addMetaNode($descr, 'description', 'text');
}
  
}

#------------------------------------------------------------------------------
# search url's in $data and place them in special format
#------------------------------------------------------------------------------

&dump_lines();

binmode STDOUT, ":utf8";

print $head->toString;
print $root->toString;
print "\n";

#--------------------------------------------------------------------------------
# get command line parameters
#--------------------------------------------------------------------------------

sub read_parse 
{
  local (*in) = @_ if @_;
  local ($i);
  push(@in, @ARGV);
  foreach $i (0 .. $#in) { $in[$i] =~ s/\+/ /g;}
  return scalar(@in);
}



