#! /usr/bin/perl
#* ============================================================
# * File  : lastfm_helper.pl
# * Author: Eric Giesselbach <ericgies@kabelfoon.nl>
# * Date  : 2007-10-04
# * Description : common subs for lastfm scripts
# * ============================================================ */
# *
# *

use English;
use XML::Simple;
use XML::DOM;
use LWP::UserAgent;
use URI;
use Time::Local;
use Digest::MD5;

$userAgent = "mythstream/0.18";
%config;
%handshake;
%state;

my $xml = new XML::Simple;
my $doc = XML::DOM::Document->new;
my $head = $doc->createXMLDecl ('1.0');
my $root = $doc->createElement('screen');

my $configfile = "lastfm/config";

sub unixtime
{
   ($sec,$min,$hour,$day,$month,$year,$wday,$yday,$isdst) = localtime(time);
   my $time = timelocal(0, 0, 0, $day, $month, $year);
   return $time;
}

sub read_config
{
   if (! -e "$configfile")
   {
      open ( FD, ">$configfile") || die "$configfile cannot be opened for writing\n";
      close FD;
   }
   
   open( FD, "<$configfile" );
   @list = <FD>;
   close FD;
  
   foreach $line (@list)
   {
     ($name, $value) = split(/\s+/, $line);
     $config{$name} = $value;
   }
}

sub write_config
{
  open ( DST, ">$configfile") || die "$configfile cannot be opened for writing\n";

  while ( ($key, $value) = each ( %config ) )
  {
    print DST "$key " . $value . "\n";
  }
  close (DST);
}

sub newNode
{
  local $name  = shift;
  local $value = shift;
  local $node = $doc->createElement($name);
  local $text = $doc->createTextNode($value);
  $node->appendChild($text);
  
  return $node;
}

sub sendItems
{
   # dump items
   print $head->toString;
   print $root->toString;

   # flush output
   print "\n";
   flush STDOUT;
}

# only line2 will be visible,
# line1 is cleared after parser exit.
sub setError
{
   local $error1 = shift;
   local $error2 = shift;

   $item = $doc->createElement('error');
   $root->appendChild($item);
   
   $item->appendChild( newNode('line1', $error1 ) );
   $item->appendChild( newNode('line2', $error2 ) );
}

# parameters: name, url, handler, pserv
sub addItem
{
   local $name    = shift;
   local $url     = shift;
   local $handler = shift;
   local $pserv   = shift;

   $item = $doc->createElement('item');
   $root->appendChild($item);
   
   $item->appendChild( newNode('name'   , $name   ) );
   $item->appendChild( newNode('url'    , $url    ) );
   $item->appendChild( newNode('handler', $handler) );
   $item->appendChild( newNode('pserv'  , $pserv  ) );
}

my $fake   = ""; # contains "fake" when valid fake url is passed
my $uName  = ""; # name of parameter in url
my $uValue = ""; # value of parameter in url


#------------------------------------------------------------------------------
# XML screen parameter output
#------------------------------------------------------------------------------

sub sendScreen
{
   # dump <screen>...</screen>
   print $head->toString;
   print $root->toString;

   # flush output
   print "\n";
   flush STDOUT;

   # new screen
   $root = $doc->createElement('screen');
}

# parameters: name, type, value
sub addField
{
   local $name  = shift;
   local $type  = shift;
   local $value = shift;

   $item = $doc->createElement('field');
   $root->appendChild($item);
   $item->appendChild( newNode('name' , $name ) );
   $item->appendChild( newNode('type' , $type ) );
   $item->appendChild( newNode('value', $value) );
}

sub decodeFakeURL
{
  local $url  = shift;    # fake url: http://fake#[name]#[value]
  
	# remove quotes if any
  $url =~ s/^\"?(.*?)\"?$/$1/;
  
  $url = substr($url, 7);
  ($fake, $uName, $uValue) = split('#', $url);

  if ($fake ne 'fake')
  {
    $uName  = "";
    $uValue = "";
  }

  return ($uName, $uValue);
}
  

sub http_lastFM_request
{
   local $url = shift;
   local %params = @_;

   $params{'session'} = $config{'session'};

   $handshake{'RESULT'} = "NOK";
   
   my $uri = URI->new( $url );
   
   $uri->query_form(\%params);

	my $ua = LWP::UserAgent->new;
	$ua->agent($userAgent); #

	my $http_header = new HTTP::Headers;
	
	my $req = HTTP::Request->new(GET => $uri);
	$req->content_type('application/x-www-form-urlencoded');

	# fetch
	my $res = $ua->request($req);

	# process response
	if ($res->is_success)
	{
      my @list = split("\n", $res->content);
      
      foreach $line (@list)
      {
        ($name, $value) = split(/=/, $line, 2);
        $handshake{$name} = $value;
      }
      
      $handshake{'RESULT'} = "OK";
	}
}

#--------------------------------------------------------------------------------
# get command line parameters
#--------------------------------------------------------------------------------

sub read_parse {
  local (*in) = @_ if @_;
  local ($i);
  push(@in, @ARGV);
  foreach $i (0 .. $#in) { $in[$i] =~ s/\+/ /g;}
  return scalar(@in);
}

return true;

