#! /usr/bin/perl
#* ============================================================
# * File  : lastfm/nowplaying.pl
# * Author: Eric Giesselbach <ericgies@kabelfoon.nl>
# * Date  : 2007-09-30
# * Description : MythStream Player Service parser
# *               connects to LastFM for nowplaying info
# * ============================================================ */
    
# * This parser is set in the stream item pserv field by lastfm/handshake.pl
# * MythStream starts this parser when that stream item reaches state "playing"
# *
# * This parser:
# *    shouldn't exit itself, MythStream will do that.
# *    should allow clean exit upon SIGxxx
# *    
# * Output (should be flushed):
# *    <screen>
# *      <field>
# *         <name>name of stream-ui field</name>
# *         <type>unused</type>
# *         <value>text (for textarea), or filename (for image)</value>
# *      <field>
# *      ...
# *    <screen>
# *
# *
# *
# *

use English;
require "./lastfm/lastfm_helper.pl";

sub reset_state
{
   $state{'command_love'} = 0;
   $state{'command_skip'} = 0;
   $state{'command_ban'} = 0;
}

sub handleButtonPress
{
   my $button = shift;
   my $command = "";

   if (!$state{'command_love'} && $button eq "button1")
     { $command = "love"; $state{'command_love'} = 1;  print STDERR "sent love\n";}
     
   if (!$state{'command_skip'} && $button eq "button2")
     { $command = "skip"; $state{'command_skip'} = 1;  print STDERR "sent skip\n";}
   
   if (!$state{'command_ban'} && $button eq "button3")
     { $command = "ban"; $state{'command_ban'} = 1;  print STDERR "sent ban\n";}

   if ($command ne "")
   {
     http_lastFM_request($control_url, ('command' => $command, 'debug' => '0') );
     
     if ( ($handshake{'RESULT'} eq "OK") )
     {
       if ($handshake{'response'} eq "OK")
       {
         addField("message", "", "Succesfully sent $command" );
         addField("command", "command", "restart" );
       }
       else
         { addField("message", "", "Sending $command failed, response: " . $handshake{'response'} ); }
     }
       else
     { 
       addField("message", "", "Connection error while sending $command" );
     }

     # pass message to player
     sendScreen();

     # get new nowplaying info in 2 seconds
     if ($command ne "love")
     {
       print STDERR "buttonpress fetch\n";
       $sleep = 2;
     }
   }
}

#------------------------------------------------------------------------------
# Init and Run
#------------------------------------------------------------------------------

# non blocking reads:
STDIN->blocking(0);

print STDERR "Hallo 1\n";

&read_config();

print STDERR "Hallo 2\n";

&read_parse();    # get commandline parameters into @in

print STDERR "Hallo 3\n";

$button = $in[0];
$button =~ s/^\"?(.*?)\"?$/$1/;

if ( ! $config{'session'} )
{
   exit(0);
}

print STDERR "Hallo 4\n";

# nowplaying query: session=<session id>
$nowplaying_url = "http://" . $config{'base_url'} . $config{'base_path'} . "/np.php";

# control query: session=<session id>, command=(love, skip, ban)
$control_url = "http://" . $config{'base_url'} . $config{'base_path'} . "/control.php";

$old_song = "";
$deltasleep = -10;
$secsinsong = 0;
$doublepoll = "";
$sleep = 1;

while (1 == 1)
{
  print STDERR "Hallo 5\n";

   http_lastFM_request($nowplaying_url, () );

print STDERR "Hallo 5a\n";

   if ( ($handshake{'RESULT'} eq "OK") && ($handshake{'streaming'} eq "true") )
   {
     print STDERR "Hallo 6\n";

      my $song = $handshake{'artist'} . " - " . $handshake{'track'};      
      my $duration = $handshake{'trackduration'};
      my $time = int($duration/60) . ':' . sprintf("%0*d", 2, $duration % 60);

      # adjust poll
      if ($old_song ne $song)  # new song started
      {
        reset_state();
        $secsinsong = 0;
        $deltasleep = -2;
        $sleep = $duration + $deltasleep;
        $doublepoll = "";
      }
        else # same song, should sleep longer
      {
        $sleep = 10;
        $doublepoll = "second poll";
      }

      $old_song = $song;
      $secsinsong += $sleep; 

      # song in "status panel" (top)
      addField("message", "", "" );
      addField("custominfo", "", $song );

      # cover-image in "dyn_panel" (mid)
      addField("logo", "", $handshake{'albumcover_medium'});

      # title of stream item is header in "audio panel" (bottom)
      addField("title", "", "LastFM Station: " . $handshake{'station'} );

      # first set of 3 rows in "audio panel"
      addField("description", "", $song );
      addField("audformat", "", "Album: " . $handshake{'album'} );
      addField("quality", "", "Duration: " . $time );

      # second set of 3 rows in "audio panel"
      addField("custom1", "", $button );
        
      # player buttons on bottom of "audio panel"
      addField("button1", "", "1. Love it" );
      addField("button2", "", "2. Next song" );
      addField("button3", "", "3. Ban the song" );
      
      sendScreen();

      print STDERR "Hallo 7\n";

      while ($sleep-- != 0)
      {
        sleep 1;
        $buttonPressed = <STDIN>;

        if (defined $buttonPressed)
        {
           handleButtonPress($buttonPressed);
        }
      }
   }
      else
   {
     print STDERR $handshake{'RESULT'} . "\n";
     sleep 30;
   }
   
}



