#! /usr/bin/perl
#* ============================================================
# * File    : menu.pl
# * Release : 2
# * Author  : Eric
# * Date    : 2008 sept
# * Description: Shoutcast genre query from disk. Source is
# *              http://yp.shoutcast.com/sbin/newxml.phtml
# *              Caching is nice and gives more actual data
# *              but now you can edit the list, it's a bit long
# * 
# * Requires mythstream v0.17_2
# * 
# * ============================================================ */

# genre list:
#   http://yp.shoutcast.com/sbin/newxml.phtml
# station list:
#   http://yp.shoutcast.com/sbin/newxml.phtml?genre=Trance
# station URL:
#   http://yp.shoutcast.com/sbin/tunein-station.pls?id=1025
# 
# This API is for noncommercial use only:
#   http://forums.winamp.com/showthread.php?s=&threadid=295638
 
    
use English;
use XML::Simple;
use XML::DOM;

$xml = new XML::Simple;

my $doc = XML::DOM::Document->new;
my $head = $doc->createXMLDecl ('1.0');
my $root = $doc->createElement('items');

sub newNode
{
  local $name  = shift;
  local $value = shift;
  local $node = $doc->createElement($name);
  local $text = $doc->createTextNode($value);
  $node->appendChild($text);
  
  return $node;
}

my $stage  = 0;  # menu level
my $choice = "";  # choice in last menu
my $path   = "";  # | separated values based on previous choices

sub decodeFakeURL
{
  local $url  = shift;    # fake url: http://fake#[stage]#[path]#[choice]

	# remove quotes if any
  $url =~ s/^\"?(.*?)\"?$/$1/;
  
  $url = substr($url, 7);
  ($fake, $stage, $path, $choice) = split('#', $url);

  if ($fake ne 'fake')
  {
    $stage  = 0;
    $choice = "";
    $path   = "";
  }

}

#------------------------------------------------------------------------------
# Init and Run
#------------------------------------------------------------------------------

  &read_parse();    # get commandline parameters into @in
  
  @alph      = split( / */, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ');
  $urlPrefix = "http://fake";
  
  $source = $in[0]; # source filename from command line
  $url    = $in[1];
  decodeFakeURL($url); # get stage, path and choice
  

  if ($stage == 0)
  {
    open( FD, "<shoutcast/genres.txt" );
    @list = <FD>;
    close FD;

    # <genre name="24hs"></genre>
  
    foreach $line (@list)
    {
      ($fill, $genre, $fill) = split('"', $line);
      $item = $doc->createElement('item');
      $root->appendChild($item);
      $item->appendChild( newNode('name' , "Shoutcast $genre") );
      $item->appendChild( newNode('url'  , "$urlPrefix#1#$genre#") );
      $item->appendChild( newNode('descr', 'Select ' . $genre ) );
      $item->appendChild( newNode('handler', '*shoutcast/menu' ) );
    }

  }
  elsif ($stage == 1)
  {
    $urlbase = 'http://yp.shoutcast.com/sbin/newxml.phtml?genre=';

    $item = $doc->createElement('item');
    $root->appendChild($item);
    $item->appendChild( newNode('name' , "Shoutcast $path") );
    $item->appendChild( newNode('url'  , $urlbase . $path . "&limit=100") );
    $item->appendChild( newNode('descr', 'Shoutcast genre $path' ) );
    $item->appendChild( newNode('handler', 'shoutcast/shoutcast' ) );
  }
  
print $head->toString;
print $root->toString;
print "\n";

#--------------------------------------------------------------------------------
# get command line parameters
#--------------------------------------------------------------------------------

sub read_parse {
  local (*in) = @_ if @_;
  local ($i);
  push(@in, @ARGV);
  foreach $i (0 .. $#in) { $in[$i] =~ s/\+/ /g;}
  return scalar(@in);
}


