#! /usr/bin/perl
#* ============================================================
# * File        : Shoutcast parser
# * Release     : 2
# * Author      : Eric
# * Date        : 2008 sept
# * Description : Parser for Shoutcast XML API
# * ============================================================ */

# genre list:
#   http://yp.shoutcast.com/sbin/newxml.phtml
# station list:
#   http://yp.shoutcast.com/sbin/newxml.phtml?genre=Trance
# station URL:
#   http://yp.shoutcast.com/sbin/tunein-station.pls?id=1025
# 
# This API is for noncommercial use only:
#   http://forums.winamp.com/showthread.php?s=&threadid=295638


use English;
use XML::DOM;
use Data::Dumper;

$xml = new XML::DOM::Parser;

my $doc = XML::DOM::Document->new;
my $head = $doc->createXMLDecl ('1.0');
my $root = $doc->createElement('items');

sub specialchars
{
   local $str = shift;
   $str =~ s/&quot;/"/g;
   $str =~ s/&amp;/&/g;

   return $str;
}

sub newNode
{
  local $name  = shift;
  local $value = shift;
  local $node = $doc->createElement($name);
  local $text = $doc->createTextNode($value);
  $node->appendChild($text);
  
  return $node;
}

sub checkNode
{
   local $node = shift;   
   return ( $node && !($node =~ /^HASH/) )  # deref empty hash element
}

sub addMetaNode
{
  local $node  = shift;
  local $name  = shift;
  local $type  = shift;
  
  if ( checkNode($node) )
  {
   if ($type eq '')
   {
     $type = "inline";
     if ($node =~ /\n/)           { $type = "text" }
     if ($node =~ /<.+>.+<\/.+>/) { $type = "html" }
   }
   
   $meta = $doc->createElement('meta');
   $meta->appendChild( newNode('name'   , $name) );
   $meta->appendChild( newNode('content', $node) );
   $meta->appendChild( newNode('viewer' , $type) );
   $item->appendChild( $meta );
  }
}

sub parseItem
{
  local $entry = shift;
  
    $url   = "http://yp.shoutcast.com/sbin/tunein-station.pls?id=" . $entry->getAttribute("id");
    $descr = $entry->getAttribute("genre") . "(" . $entry->getAttribute("br") . " kbit/s)";
    $name  = $entry->getAttribute("name");

    $name = specialchars($name);
    $descr = specialchars($descr);
    
    $item = $doc->createElement('item');
    $item->appendChild( newNode('name'   , $name));
    $item->appendChild( newNode('url'   , $url));
    $item->appendChild( newNode('descr'   , $descr) );
    
    $root->appendChild($item);
}

#------------------------------------------------------------------------------
# Init and Run
#------------------------------------------------------------------------------

&read_parse();    # get commandline parameters into @in
$source = $in[0]; # source filename from command line

eval{ $data = $xml->parsefile($source); };

$entries = $data->getElementsByTagName("station");
$n = $entries->getLength;

for(my $i=0; $i<$n; $i++)
{
	parseItem($entries->item($i));
}

binmode STDOUT, ":utf8";

print $head->toString;
print $root->toString;
print "\n";

#--------------------------------------------------------------------------------
# get command line parameters
#--------------------------------------------------------------------------------

sub read_parse {
  local (*in) = @_ if @_;
  local ($i);
  push(@in, @ARGV);
  foreach $i (0 .. $#in) { $in[$i] =~ s/\+/ /g;}
  return scalar(@in);
}


