#! /usr/bin/perl
#* ============================================================
# * File        : youtube/feed.pl
# * Release     : 1
# * Author      : Eric
# * Date        : 2006-09-11
# * Description : youtube XML feed parser
# * ============================================================ */

use English;
use XML::Simple;
use XML::DOM;
use Data::Dumper;


$xml = new XML::Simple;

my $doc = XML::DOM::Document->new;
my $head = $doc->createXMLDecl ('1.0');
my $root = $doc->createElement('items');

sub newNode
{
  local $name  = shift;
  local $value = shift;
  local $node = $doc->createElement($name);
  local $text = $doc->createTextNode($value);
  $node->appendChild($text);
  
  return $node;
}

sub checkNode
{
   local $node = shift;   
   return ( $node && !($node =~ /^HASH/) )  # deref empty hash element
}

sub addMetaNode
{
  local $node  = shift;
  local $name  = shift;
  local $type  = shift;
  
  if ( checkNode($node) )
  {
   if ($type eq '')
   {
     $type = "inline";
     if ($node =~ /\n/)           { $type = "text" }
     if ($node =~ /<.+>.+<\/.+>/) { $type = "html" }
   }
   
   $meta = $doc->createElement('meta');
   $meta->appendChild( newNode('name'   , $name) );
   $meta->appendChild( newNode('content', $node) );
   $meta->appendChild( newNode('viewer' , $type) );
   $item->appendChild( $meta );
  }
}

sub parseItem
{
  local $entry = shift;
  
  $temp = $entry->{author};
  $temp =~ s/(?:rss\@youtube.com\s)(.*)/$1/;
  
  if ($entry->{link} ne "")
  {
    $item = $doc->createElement('item');
    $item->appendChild( newNode('name'   , $entry->{title} . " $temp") );
    $item->appendChild( newNode('url'    , $entry->{link}) );
    $item->appendChild( newNode('descr'  , $entry->{pubDate}) );

    if ($playlist_enable == 1)
    {
      $item->appendChild( newNode('handler', 'youtube/vid_p') );
    }
      else
    {
      $item->appendChild( newNode('handler', 'youtube/vid') );
    }
    
    $root->appendChild($item);
  
  
    addMetaNode($entry->{description}, 'description', 'html');
    addMetaNode($entry->{author}, 'author', 'inline');
  }
}

#------------------------------------------------------------------------------
# Init and Run
#------------------------------------------------------------------------------

&read_parse();    # get commandline parameters into @in
$source = $in[0]; # source filename from command line

eval{ $data = $xml->XMLin($source); };


$channel = $data->{channel};

#print Dumper($channel->{item});

if ( ref($channel->{item}) eq 'ARRAY' )
{
  foreach my $entry (@{$channel->{item}}) { parseItem($entry); }
}
  else
{
  parseItem($channel->{item});
}

binmode STDOUT, ":utf8";

print $head->toString;
print $root->toString;
print "\n";

#--------------------------------------------------------------------------------
# get command line parameters
#--------------------------------------------------------------------------------

sub read_parse {
  local (*in) = @_ if @_;
  local ($i);
  push(@in, @ARGV);
  foreach $i (0 .. $#in) { $in[$i] =~ s/\+/ /g;}
  return scalar(@in);
}


